package com.tistory.devyongsik.moon.lucene.analysis.korean;

import java.io.IOException;
import java.util.Stack;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;

/**
 * 동의어를 SynonymEngine으로부터 가져와서 추출하는 Filter Class
 *
 * @author : 장용석, 2008. 11. 11., need4spd@google.com
 */

public class KoreanSynonymFilter extends TokenFilter {
	Log logger = LogFactory.getLog(KoreanSynonymFilter.class);

	private Stack<Token> synonymStack;
	private SynonymEngine engine;

	public KoreanSynonymFilter(TokenStream in, SynonymEngine engine) {
	    super(in);
	    synonymStack = new Stack<Token>();
	    this.engine = engine;
	}

	public KoreanSynonymFilter(TokenStream in) {
		super(in);
		synonymStack = new Stack<Token>();
		this.engine = KoreanSynonymEngine.getInstance();
	}

	public Token next(final Token reusableToken) throws IOException {
		assert reusableToken != null;

		if(logger.isDebugEnabled())
			  logger.debug("next");


		if (synonymStack.size() > 0) {
			if(logger.isDebugEnabled())
				logger.debug("동의어 Stack에서 토큰 리턴함");

			return (Token) synonymStack.pop();
		}

		Token nextToken = input.next(reusableToken);
		if (nextToken == null) {
			return null;
		}

		addAliasesToStack(nextToken);

	    //원본 Token 리턴
	    if(logger.isDebugEnabled())
			  logger.debug("원본 Token 리턴 : [" + nextToken.term() + "]");

		return nextToken;
	}

	private void addAliasesToStack(Token token) throws IOException {
		if(logger.isDebugEnabled())
			logger.debug("넘어온 Token : " + new String(token.termBuffer(), 0, token.termLength()));


		if(engine == null) {
			if(logger.isDebugEnabled())
				logger.debug("engine is null");

			return;
		}

		String[] synonyms = engine.getSynonyms(new String(token.termBuffer(), 0, token.termLength()));

		if (synonyms == null) return;

		Token synToken = null;
		for (int i = 0; i < synonyms.length; i++) {
			synToken = new Token(synonyms[i].toCharArray(), 0, synonyms[i].length(), token.startOffset(), token.endOffset());
			synToken.setPositionIncrement(0);
			synToken.setType("synonym");
			synonymStack.push(synToken);
		}
	}
}
